package com.dzdy.dim;

import com.dzdy.dim.contants.SessionPlatformEnum;
import com.dzdy.dim.handler.*;
import com.dzdy.dim.protocol.request.LoginRequestPacket;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioSocketChannel;

import java.util.concurrent.TimeUnit;

/**
 * @author : wangzhiyong
 * @date : 2019/5/23 17:38
 * description :
 */
public class NettyClient {

    /**
     * 最大连接重试次数
     */
    public static final int MAX_RETRY = 5;
    private static final int PORT = 9002;
    private static final String HOST = "127.0.0.1";
    private static Bootstrap BOOTSTRAP = new Bootstrap();

    public static void main(String[] args) {
        connect();
    }

    private static void connect() {

        NioEventLoopGroup group = new NioEventLoopGroup();

        BOOTSTRAP.group(group).channel(NioSocketChannel.class)
                // I/O 处理逻辑
                .handler(new ChannelInitializer<Channel>() {
                    @Override
                    protected void initChannel(Channel ch) {
                        ch.pipeline().addLast(new IMIdleStateHandler());
                        ch.pipeline().addLast(new Spliter());
                        ch.pipeline().addLast(new PacketDecoder());
                        // 心跳检测响应
                        ch.pipeline().addLast(new HeartBeatResponseHandler());
                        // 登录响应
                        ch.pipeline().addLast(new LoginResponseHandler());
                        // 登出响应
                        ch.pipeline().addLast(new LogoutResponseHandler());
                        // 消息接收
                        ch.pipeline().addLast(new SockDataRequestHandler());
                        ch.pipeline().addLast(new PacketEncoder());
                        ch.pipeline().addLast(new HeartBeatTimerHandler());
                    }
                });

        connect(MAX_RETRY);

        //连接超时
        BOOTSTRAP.option(ChannelOption.CONNECT_TIMEOUT_MILLIS, 5000)
                .option(ChannelOption.SO_KEEPALIVE, true)
                .option(ChannelOption.TCP_NODELAY, true);

    }

    /**
     * 登录
     * todo 登录放入token
     *
     * @param channel
     */
    private static void login(Channel channel) {
        LoginRequestPacket loginRequestPacket = new LoginRequestPacket();
        // jgzh1
//        loginRequestPacket.setToken("eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjI1NTk1NDY5MDMsInVzZXJfbmFtZSI6ImpnemgwMSIsImF1dGhvcml0aWVzIjpbIlJPTEVfT1BFUkFUT1IiXSwianRpIjoiZjA2NmQxMGMtZWVmMy00Mzk4LTg0NjctOTNiNGYwNWRiMTEwIiwiY2xpZW50X2lkIjoiYW5waW4tb3BzLXBjIiwic2NvcGUiOlsic2VsZWN0Il19.frUrkTEJaIwo-ZeMbs3L9UNmxr26WHOi52TGQWnSu00");
        // jgzh2
        loginRequestPacket.setToken("eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjI1NTk1NDY5MDMsInVzZXJfbmFtZSI6ImpnemgwMiIsImF1dGhvcml0aWVzIjpbIlJPTEVfT1BFUkFUT1IiXSwianRpIjoiZjA2NmQxMGMtZWVmMy00Mzk4LTg0NjctOTNiNGYwNWRiMTEwIiwiY2xpZW50X2lkIjoiYW5waW4tb3BzLXBjIiwic2NvcGUiOlsic2VsZWN0Il19.YnWU6sovw3-s2q4_gaFQ8j4cner2W2J-hK7MTGq_pu4");
        loginRequestPacket.setPlatform(SessionPlatformEnum.YHJ.getCode());
        loginRequestPacket.setImei("123123123123123123");
        channel.writeAndFlush(loginRequestPacket).addListener(future -> {
            if (future.isSuccess()) {
                System.out.println("client login success.");
            } else {
                System.out.println("client login failure.");
            }
        });
    }


    /**
     * 连接重试
     *
     * @param retry
     */
    public static void connect(int retry) {
        BOOTSTRAP.connect(HOST, PORT).addListener(future -> {
            if (future.isSuccess()) {
                Channel channel = ((ChannelFuture) future).channel();
                login(channel);
                System.out.println("连接成功");
            } else if (retry == 0) {
                System.err.println("重试次数已用完，放弃连接！");
            } else {
                int order = (MAX_RETRY - retry) + 1;
                // 本次重连的间隔
                int delay = 1 << order;
                System.err.println(System.currentTimeMillis() + ": 连接失败，第" + order + "次重连……");
                BOOTSTRAP.config().group().schedule(() -> connect(retry - 1), delay, TimeUnit
                        .SECONDS);
            }
        });
    }

}
