package com.dzdy.dim.handler;

import com.dzdy.dim.protocol.request.HeartBeatRequestPacket;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.EventLoop;

import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * 心跳请求定时器
 *
 * @author : wangzhiyong
 * @date : 2019/5/28 10:54
 * description :
 */
public class HeartBeatTimerHandler extends ChannelInboundHandlerAdapter {

    /**
     * 心跳间隔
     */
    private static final int HEARTBEAT_INTERVAL = 60;

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        scheduleSendHeartBeat(ctx);
        super.channelActive(ctx);
    }

    /**
     * 心跳定时发送
     *
     * @param ctx
     */
    private void scheduleSendHeartBeat(ChannelHandlerContext ctx) {
        ctx.executor().schedule(() -> {
            if (ctx.channel().isActive()) {
                System.out.println(new Date() + " send heartBeat");
                ctx.writeAndFlush(new HeartBeatRequestPacket());
                scheduleSendHeartBeat(ctx);
            }
        }, HEARTBEAT_INTERVAL, TimeUnit.SECONDS);
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        //如果运行过程中服务端挂了,执行重连机制
        EventLoop eventLoop = ctx.channel().eventLoop();
        // todo 客户端自行将服务放在容器中启动
        eventLoop.schedule(() -> null, 10L, TimeUnit.SECONDS);
        super.channelInactive(ctx);
    }
}
