package com.dzdy.dim.protocol.command;

import com.dzdy.dim.protocol.Packet;
import com.dzdy.dim.protocol.SockData;
import com.dzdy.dim.protocol.request.ConfirmRequestPacket;
import com.dzdy.dim.protocol.request.HeartBeatRequestPacket;
import com.dzdy.dim.protocol.request.LoginRequestPacket;
import com.dzdy.dim.protocol.request.LogoutRequestPacket;
import com.dzdy.dim.protocol.response.HeartBeatResponsePacket;
import com.dzdy.dim.protocol.response.LoginResponsePacket;
import com.dzdy.dim.protocol.response.LogoutResponsePacket;

import java.util.Arrays;
import java.util.Collections;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 操作指令枚举
 *
 * @author : wangzhiyong
 * @date : 2019/5/25 15:12
 * description :
 */
public enum Command {

    /* ================= 基础指令 ================= */

    /**
     * 登录请求
     */
    LOGIN_REQUEST(1, LoginRequestPacket.class),
    /**
     * 登录返回
     */
    LOGIN_RESPONSE(2, LoginResponsePacket.class),
    /**
     * 登出请求
     */
    LOGOUT_REQUEST(3, LogoutRequestPacket.class),
    /**
     * 登出返回
     */
    LOGOUT_RESPONSE(4, LogoutResponsePacket.class),
    /**
     * 消息确认（ACK）
     */
    CONFIRM(5, ConfirmRequestPacket.class),
    /**
     * 心跳请求
     */
    HEARTBEAT_REQUEST(6, HeartBeatRequestPacket.class),
    /**
     * 心跳返回
     */
    HEARTBEAT_RESPONSE(7, HeartBeatResponsePacket.class),


    /* ================= 业务指令 ================= */

    /**
     * 人员数据同步
     */
    PERSON(90, SockData.class),
    /**
     * 消息推送
     */
    MESSAGE(91, SockData.class);

    /**
     * 数据 map, 用于加快查找, O(n) -> O(1)
     */
    private static final Map<Byte, Command> DATA_MAP;

    static {
        DATA_MAP = Collections.unmodifiableMap(Arrays.stream(Command.values())
                .collect(Collectors.toMap(Command::getCode, commandEnum -> commandEnum)));
    }

    private Byte code;
    private Class<? extends Packet> clazz;

    Command(int code, Class<? extends Packet> clazz) {
        this.code = (byte) code;
        this.clazz = clazz;
    }

    public static Command getCommand(byte command) {
        return DATA_MAP.get(command);
    }

    public Byte getCode() {
        return code;
    }

    public Class<? extends Packet> getClazz() {
        return clazz;
    }
}
