package com.dzdy.dim.util;

import com.dzdy.dim.contants.AttrContants;
import com.dzdy.dim.contants.Session;
import io.netty.channel.Channel;
import io.netty.util.Attribute;
import io.netty.util.internal.StringUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * session 管理
 * todo 后续拓展其他存储方式 redis/mysql 等
 *
 * @author : wangzhiyong
 * @date : 2019/5/27 13:56
 * description :
 */
public class SessionUtil {

    private static final Map<String, Channel> USER_ID_CHANNEL_MAP = new ConcurrentHashMap<>();

    /**
     * 绑定session和通道
     * @param session
     * @param channel
     */
    public static void bind(Session session, Channel channel) {
        if (!hasLogin(channel)) {
            USER_ID_CHANNEL_MAP.put(session.getSessionId(), channel);
            channel.attr(AttrContants.SESSION).set(session);
        }
    }

    /**
     * 解除通道绑定
     * @param channel
     */
    public static void unBind(Channel channel) {
        Session session = getSession(channel);
        if (session != null) {
            USER_ID_CHANNEL_MAP.remove(session.getSessionId());
            channel.attr(AttrContants.SESSION).set(null);
        }
    }

    /**
     * 判断通道登录状态
     * @param channel
     * @return
     */
    public static boolean hasLogin(Channel channel) {
        return getSession(channel) != null;
    }

    /**
     * 通过通道获取session
     * @param channel
     * @return
     */
    public static Session getSession(Channel channel) {
        Attribute<Session> attr = channel.attr(AttrContants.SESSION);
        if (attr != null) {
            return attr.get();
        }
        return null;
    }

    /**
     * 通过session获取通道
     * @param sessionId
     * @return
     */
    public static Channel getChannel(String sessionId) {
        return USER_ID_CHANNEL_MAP.get(sessionId);
    }

    /**
     * 通过用户id获取session
     * @param userId
     * @return
     */
    public static List<Session> getSession(String userId) {
        List<Session> sessionList = new ArrayList<>(USER_ID_CHANNEL_MAP.size());
        for (Channel channel : USER_ID_CHANNEL_MAP.values()) {
            Session session = getSession(channel);
            if (session != null) {
                if (!StringUtil.isNullOrEmpty(userId) && !userId.equals(session.getUserId())) {
                    continue;
                }
                sessionList.add(session);
            }
        }
        return sessionList;
    }
}
