package com.dzdy.dim.config;

import com.dzdy.dim.handler.*;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioSocketChannel;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

/**
 * socket 配置
 *
 * @author : wangzhiyong
 * @date : 2019/5/30 10:34
 * description :
 */
@Data
@Component
@ConfigurationProperties(prefix = "dim.socket")
public class SocketConfig {

    /**
     * boss 线程数 一般为1
     */
    private Integer boosCount = 1;
    /**
     * worker 线程数 默认为核心数*2
     */
    private Integer workerCount;
    /**
     * 监听端口
     */
    private Integer port = 9001;
    /**
     * 握手请求的队列的最大长度
     */
    private Integer soBackLog = 1024;

    @Bean("serverBootstrap")
    public ServerBootstrap serverBootstrap() {
        return new ServerBootstrap();
    }

    @Bean("bossGroup")
    public NioEventLoopGroup bossGroup() {
        return new NioEventLoopGroup(boosCount);
    }

    @Bean("workerGroup")
    public NioEventLoopGroup workerGroup() {
        if (workerCount != null) {
            return new NioEventLoopGroup(workerCount);
        }
        return new NioEventLoopGroup();
    }

    @Bean("channelHandler")
    public ChannelHandler channelHandler() {
        return new ChannelInitializer<NioSocketChannel>() {
            @Override
            protected void initChannel(NioSocketChannel ch) {
                ch.pipeline().addLast(new IMIdleStateHandler());
                ch.pipeline().addLast(new Spliter());
                ch.pipeline().addLast(PacketCodecHandler.INSTANCE);
                ch.pipeline().addLast(LoginRequestHandler.INSTANCE);
                ch.pipeline().addLast(HeartBeatRequestHandler.INSTANCE);
                ch.pipeline().addLast(AuthHandler.INSTANCE);
                ch.pipeline().addLast(LogoutRequestHandler.INSTANCE);
                ch.pipeline().addLast(ConfirmRequestHandler.INSTANCE);
            }
        };
    }
}
