package com.dzdy.dim.config;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelOption;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PreDestroy;
import javax.annotation.Resource;

/**
 * socket 服务管理
 *
 * @author : wangzhiyong
 * @date : 2019/5/30 11:24
 * description :
 */
@Slf4j
@Configuration
public class SocketService {

    @Resource
    private SocketConfig config;
    @Resource
    private ServerBootstrap serverBootstrap;
    @Resource
    private NioEventLoopGroup bossGroup;
    @Resource
    private NioEventLoopGroup workerGroup;
    @Resource
    private ChannelHandler channelHandler;

    public void run() {
        serverBootstrap
                .group(bossGroup, workerGroup)
                //指定我们服务端的 IO 模型为NIO
                .channel(NioServerSocketChannel.class)
                .option(ChannelOption.SO_BACKLOG, config.getSoBackLog())
                // TCP 心跳机制 true 开启
                .childOption(ChannelOption.SO_KEEPALIVE, true)
                // 是否关闭Nagle算法 true 高实时性，有消息立即发送 false 减少发送次数减少网络交互 提升性能(Nagle 算法)
                .childOption(ChannelOption.TCP_NODELAY, true)
                .childHandler(channelHandler);
        serverBootstrap.bind(config.getPort()).addListener(future -> {
            if (future.isSuccess()) {
                log.info("socket service port [" + config.getPort() + "] bind success . ");
            } else {
                log.error("socket service port [" + config.getPort() + "] bind failure .");
            }
        });
    }

    @PreDestroy
    public void stop() {
        bossGroup.shutdownGracefully();
        workerGroup.shutdownGracefully();
    }
}
