package com.dzdy.dim.config;

import com.dzdy.dim.handler.IMIdleStateHandler;
import com.dzdy.dim.handler.websocket.WebSocketIndexPageHandler;
import com.dzdy.dim.handler.websocket.WebSocketPacketCodecHandler;
import com.dzdy.dim.handler.websocket.WebSocketSocketDateHandler;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpServerCodec;
import io.netty.handler.codec.http.websocketx.WebSocketServerProtocolHandler;
import io.netty.handler.ssl.SslContext;
import io.netty.handler.ssl.SslContextBuilder;
import io.netty.handler.ssl.util.SelfSignedCertificate;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

/**
 * websocket 配置
 *
 * @author : wangzhiyong
 * @date : 2019/5/30 10:34
 * description :
 */
@Data
@Component
@ConfigurationProperties(prefix = "dim.websocket")
public class WebSocketConfig {

    /**
     * boss 线程数 一般为1
     */
    private Integer boosCount = 1;
    /**
     * worker 线程数 默认为核心数*2
     */
    private Integer workerCount;
    /**
     * 监听端口
     */
    private Integer port = 9010;

    /**
     * 是否开启ssl
     */
    private Boolean ssl = false;

    /**
     * sslContext
     */
    private SslContext sslCtx;

    /**
     * websocket 前缀地址
     */
    private String websocketPath = "/websocket";

    @PostConstruct
    public void init() throws Exception {
        if (ssl) {
            SelfSignedCertificate ssc = new SelfSignedCertificate();
            sslCtx = SslContextBuilder.forServer(ssc.certificate(), ssc.privateKey()).build();
        } else {
            sslCtx = null;
        }
    }

    @Bean("webServerBootstrap")
    public ServerBootstrap serverBootstrap() {
        return new ServerBootstrap();
    }

    @Bean("webBossGroup")
    public EventLoopGroup bossGroup() {
        return new NioEventLoopGroup(boosCount);
    }

    @Bean("webWorkerGroup")
    public EventLoopGroup workerGroup() {
        if (workerCount != null) {
            return new NioEventLoopGroup(workerCount);
        }
        return new NioEventLoopGroup();
    }

    @Bean("webChannelHandler")
    public ChannelHandler channelHandler() {
        return new ChannelInitializer<SocketChannel>() {
            @Override
            protected void initChannel(SocketChannel ch) throws Exception {
                ChannelPipeline pipeline = ch.pipeline();
                if (sslCtx != null) {
                    pipeline.addLast(sslCtx.newHandler(ch.alloc()));
                }
                pipeline.addLast(new IMIdleStateHandler());
                pipeline.addLast(new HttpServerCodec());
                pipeline.addLast(new HttpObjectAggregator(65536));
                pipeline.addLast(new WebSocketServerProtocolHandler(websocketPath, null, true));
                pipeline.addLast(WebSocketIndexPageHandler.INSTANCE);
                pipeline.addLast(WebSocketPacketCodecHandler.INSTANCE);
                pipeline.addLast(WebSocketSocketDateHandler.INSTANCE);
            }
        };
    }
}
