package com.dzdy.dim.config;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PreDestroy;
import javax.annotation.Resource;

/**
 * websocket 服务管理
 *
 * @author : wangzhiyong
 * @date : 2019/6/18 10:52
 * description :
 */
@Slf4j
@Configuration
public class WebSocketServer {

    @Resource
    private WebSocketConfig config;
    @Resource
    private ServerBootstrap webServerBootstrap;
    @Resource
    private EventLoopGroup webBossGroup;
    @Resource
    private EventLoopGroup webWorkerGroup;
    @Resource
    private ChannelHandler webChannelHandler;

    public void run() throws Exception {
        webServerBootstrap.group(webBossGroup, webWorkerGroup)
                .channel(NioServerSocketChannel.class)
                .childHandler(webChannelHandler);
        Channel channel = webServerBootstrap.bind(config.getPort()).addListener(future -> {
            if (future.isSuccess()) {
                log.info("websocket service port [{}] bind success . ", config.getPort());
            } else {
                log.error("websocket service port [{}] bind failure .", config.getPort());
            }
        }).sync().channel();
        log.info("Open your web browser and navigate to " + "http://127.0.0.1:" + config.getPort() + '/');
        channel.closeFuture().sync();
    }

    @PreDestroy
    public void stop() {
        webBossGroup.shutdownGracefully();
        webWorkerGroup.shutdownGracefully();
    }
}
