package com.dzdy.dim.handler;

import com.dzdy.dim.contants.Session;
import com.dzdy.dim.protocol.command.JWTUser;
import com.dzdy.dim.protocol.request.LoginRequestPacket;
import com.dzdy.dim.protocol.response.LoginResponsePacket;
import com.dzdy.dim.util.ApplicationContextUtils;
import com.dzdy.dim.util.JWTUtil;
import com.dzdy.dim.util.SessionUtil;
import io.jsonwebtoken.ExpiredJwtException;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;

/**
 * 登录请求处理器
 *
 * @author : wangzhiyong
 * @date : 2019/5/28 14:38
 * description :
 */
@Slf4j
@ChannelHandler.Sharable
public class LoginRequestHandler extends SimpleChannelInboundHandler<LoginRequestPacket> {

    public static final LoginRequestHandler INSTANCE = new LoginRequestHandler();

    private final JWTUtil jwtUtil;

    private LoginRequestHandler() {
        jwtUtil = ApplicationContextUtils.getBean(JWTUtil.class);
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, LoginRequestPacket packet) {
        LoginResponsePacket responsePacket = new LoginResponsePacket();
        responsePacket.setVersion(packet.getVersion());

        String token = packet.getToken();
        try {
            JWTUser user = jwtUtil.decode(token);
            if (null != user) {
                responsePacket.setSuccess(true);
                Session session = Session.buildSession(user.getUser_name(), packet.getPlatform(), packet.getImei());
                responsePacket.setSession(session);
                SessionUtil.bind(session, ctx.channel());
                log.info("[user:{} platform:{}] login success .", user.getUser_name(), packet.getPlatform());
            }
        } catch (ExpiredJwtException e) {
            responsePacket.setSuccess(false);
            responsePacket.setReason("token过期，连接失败");
        } catch (Exception e) {
            log.error("token:[" + token + "]解析异常", e);
            responsePacket.setSuccess(false);
            responsePacket.setReason("token错误，连接失败");
        }
        ctx.writeAndFlush(responsePacket);
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) {
        log.info("有客户端断开连接");
        SessionUtil.unBind(ctx.channel());
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        log.error("socket service exception:", cause);
        ctx.close();
    }
}
