package com.dzdy.dim.handler.websocket;

import com.dzdy.dim.protocol.Packet;
import com.dzdy.dim.protocol.SockData;
import com.dzdy.dim.serialize.Serializer;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToMessageCodec;
import io.netty.handler.codec.http.websocketx.TextWebSocketFrame;
import io.netty.handler.codec.http.websocketx.WebSocketFrame;

import java.util.List;

/**
 * @author : wangzhiyong
 * @date : 2019/6/20 13:56
 * description :
 */
@ChannelHandler.Sharable
public class WebSocketPacketCodecHandler extends MessageToMessageCodec<WebSocketFrame, Packet> {

    public static final WebSocketPacketCodecHandler INSTANCE = new WebSocketPacketCodecHandler();

    private static final Serializer SERIALIZER = Serializer.DEFAULT;

    private WebSocketPacketCodecHandler() {
    }

    @Override
    protected void encode(ChannelHandlerContext ctx, Packet data, List<Object> out) throws Exception {
        out.add(new TextWebSocketFrame(SERIALIZER.serialize(data, false)));
    }

    @Override
    protected void decode(ChannelHandlerContext ctx, WebSocketFrame frame, List<Object> out) throws Exception {
        if (frame instanceof TextWebSocketFrame) {
            String text = ((TextWebSocketFrame) frame).text();
            out.add(SERIALIZER.deserialize(text.getBytes(), SockData.class));
        } else {
            String message = "unsupported frame type: " + frame.getClass().getName();
            throw new UnsupportedOperationException(message);
        }
    }
}
