package com.dzdy.dim.handler.websocket;

import com.dzdy.dim.contants.Session;
import com.dzdy.dim.contants.SessionPlatformEnum;
import com.dzdy.dim.protocol.SockData;
import com.dzdy.dim.protocol.command.Command;
import com.dzdy.dim.protocol.command.JWTUser;
import com.dzdy.dim.protocol.response.LoginResponsePacket;
import com.dzdy.dim.util.ApplicationContextUtils;
import com.dzdy.dim.util.JWTUtil;
import com.dzdy.dim.util.SessionUtil;
import io.jsonwebtoken.ExpiredJwtException;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;

/**
 * @author : wangzhiyong
 * @date : 2019/6/20 11:19
 * description :
 */
@Slf4j
@ChannelHandler.Sharable
public class WebSocketSocketDateHandler extends SimpleChannelInboundHandler<SockData> {

    public static final WebSocketSocketDateHandler INSTANCE = new WebSocketSocketDateHandler();

    private final JWTUtil jwtUtil;

    private WebSocketSocketDateHandler() {
        jwtUtil = ApplicationContextUtils.getBean(JWTUtil.class);
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, SockData sockData) throws Exception {
        Command command = Command.getCommand(sockData.getCommand());
        Object data = sockData.getData();
        switch (command) {
            case LOGIN_REQUEST:
                LoginResponsePacket responsePacket = new LoginResponsePacket();
                String token = data.toString();
                try {
                    JWTUser user = jwtUtil.decode(token);
                    if (null != user) {
                        responsePacket.setSuccess(true);
                        Session session = Session.buildSession(user.getUser_name(), SessionPlatformEnum.WEB.getCode());
                        responsePacket.setSession(session);
                        SessionUtil.bind(session, ctx.channel());
                        log.info("[user:{} platform:{}] login success .", user.getUser_name(), SessionPlatformEnum.WEB.getCode());
                    }
                } catch (ExpiredJwtException e) {
                    responsePacket.setSuccess(false);
                    responsePacket.setReason("token过期，连接失败");
                } catch (Exception e) {
                    log.error("token:[" + token + "]解析异常", e);
                    responsePacket.setSuccess(false);
                    responsePacket.setReason("token错误，连接失败");
                }
                ctx.writeAndFlush(responsePacket);
                break;
            case CONFIRM:
                log.info("messageId:{} is consumed", data.toString());
                break;
            default:
                // 授权检查 清除非正常连接
                if (!SessionUtil.hasLogin(ctx.channel())) {
                    ctx.channel().close();
                }
        }
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) {
        log.info("有客户端断开连接");
        SessionUtil.unBind(ctx.channel());
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        log.error("websocket service exception:", cause);
        ctx.close();
    }
}
