package com.dzdy.dim.util;

import com.alibaba.fastjson.JSONObject;
import com.dzdy.dim.protocol.command.JWTUser;
import io.jsonwebtoken.*;
import io.jsonwebtoken.security.InvalidKeyException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

/**
 * jwt 校验
 *
 * @author : wangzhiyong
 * @date : 2019/5/28 14:45
 * description :
 */
@Slf4j
@Component
public class JWTUtil {

    @Value("${dim.jwt.secret}")
    private String key;
    private SecretKey secretKey;

    @PostConstruct
    private void init() {
        secretKey = new SecretKeySpec(key.getBytes(), SignatureAlgorithm.HS256.getJcaName());
    }

    /**
     * token 解密
     *
     * @param token
     * @return
     * @throws ExpiredJwtException
     */
    public JWTUser decode(String token) throws ExpiredJwtException {
        if (null != token && token.length() > 0) {
            Jws<Claims> jws = Jwts.parser().setSigningKey(secretKey).parseClaimsJws(token);
            Claims body = jws.getBody();
            return JSONObject.parseObject(JSONObject.toJSONString(body), JWTUser.class);
        }
        return null;
    }

    /**
     * token 加密
     *
     * @param object
     * @return
     */
    public String encode(String object) {
        try {
            return Jwts.builder().setSubject(object).signWith(secretKey).compact();
        } catch (InvalidKeyException e) {
            log.error("JWTUtil - encode", e);
        }
        return null;
    }

}
